// --- START OF FILE VESCTelemetry_V1.38_FasterUpdate.ino --- //

// CURRENT PROJECT
// Vibe coded by David Bloomfield using Gemini 2.5 Pro Exp, inspired by Lukas Janky's work
// last edit on 2025.04.21 (Gemini Mod)
// Gemini 2nd Mod (2025-04-21): Fix layout Screen 4 (Dist label, Avg Cons units), Fix Settings highlight.
// Gemini 3rd Mod (2025-04-21): Implement conditional EEPROM save logic, adjust Screen 4 layout further.
// Gemini 4th Mod (2025-04-21): Correct distance calc factor (3.0->6.0), adjust Screen 4 unit layout again.
// Gemini 5th Mod (2025-04-21): Add explicit clear rect for speed display on Screen 1 to fix artifacts.
// Gemini 6th Mod (2025-04-22): Optimize Screen 1 speed display to redraw only on value change, reducing flicker.
// Gemini 7th Mod (2025-04-22): Add regen color indication, reorder Screen 2, apply conditional draw to more elements.
// Gemini 8th Mod (2025-04-22): Fix compilation errors due to incorrect color define names.
// Gemini 9th Mod (2025-04-22): Revert extensive conditional drawing to fix RAM-related boot loop. Keep speed optimization.
// Gemini 10th Mod(2025-04-22): Increase screenUpdateInterval to 100ms (10Hz) to reduce flicker and improve button responsiveness.
// Gemini 11th Mod(2025-04-22): Decrease screenUpdateInterval to 33ms (~30Hz) by user request.
// Changes: Rewrote Screen 3 drawing without helper, drawing all text with BG color
//          to try and fix label flicker specific to that screen.
//          Includes previous fixes. Added: Global constants, Odometer (EEPROM), Screen 4 redesign, Settings update, Pole pair fix, Power display fix.
// use with st7789 display

#include <VescUart.h>
#include <EEPROM.h> // Include EEPROM library for persistence

//Library for the Display
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Arduino_ST7789_Fast.h> // Using the Fast library is good!

//--------------------------------------------------------------------------------------------------
// --- GLOBAL CONFIGURATION CONSTANTS (ADJUST THESE FOR YOUR SETUP) ---
//--------------------------------------------------------------------------------------------------

// --- System / Hardware ---
const int POLE_PAIRS = 3;             // <<< Number of motor pole pairs (physical RPM = eRPM / POLE_PAIRS)
const float WHEEL_DIAMETER_M = 0.2794; // Wheel diameter in meters (e.g., 11 inches * 0.0254 m/inch)
const float GEAR_RATIO = 7.056;        // Motor pulley teeth / Wheel pulley teeth (or 1.0 if direct drive)

// --- Battery ---
const int CELL_COUNT = 10;          // Number of cells in series in your battery pack
const float MIN_VOLTAGE = 34.0;     // Voltage at 0% battery (e.g., 3.4V * CELL_COUNT)
const float MAX_VOLTAGE = 42.0;     // Voltage at 100% battery (e.g., 4.2V * CELL_COUNT)

// --- Odometer ---
const unsigned long ODO_SAVE_MIN_INTERVAL = 240000UL; // Min time before enabling save check (240 seconds = 4 minutes)
const float ODO_SAVE_SPEED_THRESHOLD_KMH = 2.0; // Speed threshold (in km/h) below which saving is allowed after timer elapsed

// --- EEPROM Addresses ---
const int EEPROM_ADDR_UNITS = 0;       // 1 byte (0 or 1)
const int EEPROM_ADDR_ROTATION = 1;    // 1 byte (0-3)
const int EEPROM_ADDR_ODOMETER = 2;    // Start address for odometer float (uses sizeof(float) bytes, typically 4)
const int EEPROM_SIZE = 16;            // Minimum EEPROM size needed

//--------------------------------------------------------------------------------------------------
// --- END OF GLOBAL CONFIGURATION CONSTANTS ---
//--------------------------------------------------------------------------------------------------

//#define TFT_CS    6 // Keep commented if using hardware CS
#define TFT_DC 9
#define TFT_RST 8
#define SCR_WD 240
#define SCR_HT 240
Arduino_ST7789 lcd = Arduino_ST7789(TFT_DC, TFT_RST);
//Arduino_ST7789 lcd = Arduino_ST7789(TFT_DC, TFT_RST, TFT_CS); // Uncomment if using software CS on pin 6

// Colors
#define Black 0x0000       /* 0,   0,   0 */
#define Navy 0x000F        /* 0,   0, 128 */
#define DarkGreen 0x03E0   /* 0, 128,   0 */
#define DarkCyan 0x03EF    /* 0, 128, 128 */
#define Maroon 0x7800      /* 128,   0,   0 */
#define Purple 0x780F      /* 128,   0, 128 */
#define Olive 0x7BE0       /* 128, 128,   0 */
#define LightGrey 0xD69A   /* 211, 211, 211 */
#define DarkGrey 0x7BEF    /* 128, 128, 128 */
#define Blue 0x001F        /* 0,   0, 255 */
#define Green 0x07E0       /* 0, 255,   0 */ // Used for Regen indication
#define Cyan 0x07FF        /* 0, 255, 255 */
#define Red 0xF800         /* 255,   0,   0 */ // Default Label/Unit color
#define Magenta 0xF81F     /* 255,   0, 255 */
#define Yellow 0xFFE0      /* 255, 255,   0 */
#define White 0xFFFF       /* 255, 255, 255 */ // Default Value color
#define Orange 0xFD20      /* 255, 165,   0 */
#define GreenYellow 0xAFE5 /* 173, 255,  47 */
#define Pink 0xF81F

// Define custom colors
#define DEFAULT_LABEL_COLOR Red
#define DEFAULT_VALUE_COLOR White
#define DEFAULT_UNIT_COLOR Red
#define REGEN_COLOR Green // Color for labels/units during regen
#define SETTINGS_ITEM_COLOR White
#define SETTINGS_FOCUS_COLOR White
#define SETTINGS_FOCUS_BG_COLOR DarkGrey
#define SETTINGS_SELECTED_COLOR White
#define SEPARATOR_DOT_COLOR DarkGrey
#define TITLE_SEPARATOR_COLOR White

/** Initiate VescUart class */
VescUart UART;

// VESC Data Variables (Raw Metric)
int rpm = 0;
float voltage = 0.0;
float current = 0.0; // Battery current
float phase_current = 0.0; // Phase current
int power = 0;
float amphour = 0.0;
float tach = 0.0;
float distance = 0.0; // Current trip distance
float velocity = 0.0; // INSTANTANEOUS velocity in km/h (RAW)
float watthour = 0.0;
float batpercentage = 0.0;
float temp_fet = 0.0;
float temp_motor = 0.0;
float whkm = 0.0; // Raw Wh/km

// Odometer Variables
float odometer = 0.0;
float lastDistanceOdo = 0.0;
unsigned long odoTimerStartTime = 0;
bool odoTimerElapsed = false;
bool odoSavePending = false;

// Display Variables (potentially converted)
float displayVelocity = 0.0;
float displayDistance = 0.0;
float displayOdometer = 0.0;
float displayWhPerDist = 0.0;
float displayTempFet = 0.0;
float displayTempMotor = 0.0;
float displayMaxFetTemp = 0.0;
float displayMaxMotorTemp = 0.0;
char velocityUnitStr[5] = "km/h";
char distanceUnitStr[4] = "km";
char odoUnitStr[4] = "km";
char tempUnitStr[2] = "C";
char consumptionUnitStr[6] = "Wh/km";

// --- State variable ONLY for conditional speed drawing ---
const float UNINITIALIZED_FLOAT = -999.9f;
float lastDispVelocity1 = UNINITIALIZED_FLOAT; // Track only speed for Screen 1 optimization

// State variables
#define DISPLAY_STATE_MAX 4
#define SETTINGS_STATE 0
int currentState = 1;
int lastState = -1;
bool vescDataUpdated = false;
unsigned long lastButtonPressTime = 0;
const unsigned long LONG_PRESS_DURATION = 1000;

// Settings state variables
#define NUM_SETTINGS_ITEMS 9
#define RESERVED_FOCUS_INDEX 6
#define CLEAR_ODO_FOCUS_INDEX 7
#define EXIT_FOCUS_INDEX 8
int settingsFocus = 0;
int currentUnits = 0;
int currentRotation = 2;

// Button timing variables
bool buttonIsDown = false;
unsigned long buttonDownTime = 0;
bool longPressActive = false;

// Screen Timeout Variables
unsigned long lastDataUpdateTime = 0;
const unsigned long screenTimeoutDuration = 60000UL;
bool isScreenOff = false;

// Screen Update Throttling Variables
unsigned long lastScreenUpdateTime = 0;
// *** Decreased interval for faster updates (~30Hz) ***
const unsigned long screenUpdateInterval = 33; // Was 100ms, originally 50ms

// Maximum values (Raw Metric)
float maxa = 0.0;
int maxvel = 0;
float maxfetemp = -999.0f;
float maxmotortemp = -999.0f;

// Conversion Factors
const float KM_TO_MILES = 0.621371;
const float MILES_TO_KM = 1.60934;

// Alignment constants
const int labelStartX1 = 15;
const int valueStartX1 = 145;
const int unitStartX1 = 205;
// const int valueWidth1 = 60; // No longer needed with simplified drawing
const int velocity_y = 45;
const int velocity_height = 75;
const int labelStartX2 = 10;
const int valueStartX2 = 155;
const int unitStartX2 = 210;
// const int valueWidth2 = 65; // No longer needed with simplified drawing
const int labelStartX3 = 15;
const int valueStartX3 = 150;
const int settingsLabelX = 15;
const int settingsOptionIndent = 30;
const int settingsUnitsOptionX = settingsOptionIndent;
const int settingsUnitsOptionSpacing = 100;
const int settingsRotOptionX = settingsOptionIndent;
const int settingsRotOptionSpacing = 50;
const int rowSeparatorMargin = 5;
const int separatorDotSpacing = 4;
const int separatorStartX = 25;
const int separatorEndX = SCR_WD - separatorStartX;
const int titleSeparatorHeight = 3;

// Function Prototypes
void drawScreen1(); // Removed fullRedraw flag - handled internally for speed
void drawScreen2(); // Simplified - redraws all elements
void drawScreen3(); // Simplified - redraws all elements
void drawScreen4(); // Simplified - redraws all elements
void drawSettingsScreen(); // Settings redraws fully on focus change
void drawDottedLine(int x0, int y, int x1, uint16_t color, int spacing);
void updateDisplayValues();
void saveOdometer();
// Removed resetLastDisplayedValues - only tracking speed now

//SETUP...............................................................................................................................................................
void setup() {
  Serial.begin(115200);
  lcd.init(SCR_WD, SCR_HT);

  // --- Load Settings & Odometer ---
  // EEPROM.begin(EEPROM_SIZE); // Uncomment if needed

  byte storedUnits = EEPROM.read(EEPROM_ADDR_UNITS);
  byte storedRotation = EEPROM.read(EEPROM_ADDR_ROTATION);
  EEPROM.get(EEPROM_ADDR_ODOMETER, odometer);

  if (isnan(odometer) || odometer < 0.0f || odometer > 999999.0f) {
      odometer = 0.0f;
      EEPROM.put(EEPROM_ADDR_ODOMETER, odometer);
      // EEPROM.commit(); // Uncomment if needed
  }
  lastDistanceOdo = 0.0;

  currentUnits = (storedUnits > 1) ? 0 : storedUnits;
  if (storedUnits > 1) EEPROM.write(EEPROM_ADDR_UNITS, currentUnits);

  currentRotation = (storedRotation > 3) ? 2 : storedRotation;
  if (storedRotation > 3) EEPROM.write(EEPROM_ADDR_ROTATION, currentRotation);

  // EEPROM.commit(); // Uncomment if needed

  lcd.setRotation(currentRotation);

  // --- Initial Display ---
  lcd.fillScreen(BLACK);
  lcd.setCursor(10, 100); lcd.setTextColor(WHITE); lcd.setTextSize(3);
  lcd.println(F("INITIALIZING"));
  lcd.setCursor(85, 130); lcd.print(F("VESC"));

  UART.setSerialPort(&Serial);
  pinMode(2, INPUT_PULLUP);

  delay(1500);

  // Initialize state variables
  lastState = -1;
  currentState = 1;
  isScreenOff = false;
  lastDataUpdateTime = millis();
  lastScreenUpdateTime = millis();
  odoTimerStartTime = millis();
  odoTimerElapsed = false;
  odoSavePending = false;
  lastDispVelocity1 = UNINITIALIZED_FLOAT; // Initialize only speed tracking

  lcd.fillScreen(BLACK);
  updateDisplayValues();
  drawScreen1(); // Draw screen 1 (will force full draw internally first time)
  lastState = currentState;
}


// Function to handle saving Odometer and resetting flags/timer
void saveOdometer() {
    // Serial.println("Saving Odometer..."); // Debug message
    EEPROM.put(EEPROM_ADDR_ODOMETER, odometer);
    // EEPROM.commit(); // Uncomment if needed

    odoSavePending = false;
    odoTimerElapsed = false;
    odoTimerStartTime = millis();
}

//LOOP START............................................................................................................................................................
void loop() {
  vescDataUpdated = false;
  bool shortPressTrigger = false;
  bool longPressTrigger = false;
  unsigned long currentMillis = millis();

  ////////// Read values from VESC //////////
  if (UART.getVescValues()) {
    vescDataUpdated = true;
    lastDataUpdateTime = currentMillis;

    // --- Assign RAW metric values ---
    rpm = (UART.data.rpm) / POLE_PAIRS;
    voltage = (UART.data.inpVoltage);
    current = (UART.data.avgInputCurrent); // Batt A
    phase_current = (UART.data.avgMotorCurrent); // Phase A
    power = voltage * current; // Power W
    temp_fet = (UART.data.tempMosfet);
    temp_motor = (UART.data.tempMotor);
    amphour = (UART.data.ampHours);
    watthour = amphour * voltage;
    tach = (UART.data.tachometerAbs);

    // --- Calculate RAW Metric derived values ---
    const float M_PER_KM = 1000.0;
    const float SEC_PER_MIN = 60.0;
    const float PULSES_PER_MOTOR_REV = POLE_PAIRS * 6.0; // Corrected factor

    float motor_revolutions = (PULSES_PER_MOTOR_REV > 0) ? UART.data.tachometerAbs / PULSES_PER_MOTOR_REV : 0;
    velocity = (rpm / GEAR_RATIO) * PI * WHEEL_DIAMETER_M * (SEC_PER_MIN / M_PER_KM);
    distance = (motor_revolutions / GEAR_RATIO) * PI * WHEEL_DIAMETER_M / M_PER_KM;

    const float VOLTAGE_RANGE = MAX_VOLTAGE - MIN_VOLTAGE;
    float clamped_voltage = constrain(voltage, MIN_VOLTAGE, MAX_VOLTAGE);
    if (VOLTAGE_RANGE > 0) { batpercentage = ((clamped_voltage - MIN_VOLTAGE) / VOLTAGE_RANGE) * 100.0; }
    else { batpercentage = (voltage >= MAX_VOLTAGE) ? 100.0 : 0.0; }
    batpercentage = constrain(batpercentage, 0.0, 100.0);

    if (current > maxa) { maxa = current; }
    int current_velocity_int = round(velocity);
    if (current_velocity_int > maxvel) { maxvel = current_velocity_int; }
    if (temp_fet > maxfetemp) { maxfetemp = temp_fet; }
    if (temp_motor > maxmotortemp) { maxmotortemp = temp_motor; }

    if (distance > 0.01) { whkm = watthour / distance; } else { whkm = 0.0; }

    // --- Update Odometer ---
    if (vescDataUpdated && distance > lastDistanceOdo && distance < (lastDistanceOdo + 100.0)) {
        float distanceIncrement = distance - lastDistanceOdo;
        odometer += distanceIncrement;
    }
    if (vescDataUpdated) { lastDistanceOdo = distance; }

    // --- Check Odometer Save Condition (Speed) ---
    if (odoTimerElapsed && !odoSavePending && (velocity < ODO_SAVE_SPEED_THRESHOLD_KMH)) {
        odoSavePending = true;
    }

    // --- Update Display Variables (if not in settings) ---
    if (currentState != SETTINGS_STATE) {
        updateDisplayValues();
    }

  } // End VESC Data Read

  // --- Check Odometer Save Condition (Timer) ---
  if (!odoTimerElapsed && (currentMillis - odoTimerStartTime >= ODO_SAVE_MIN_INTERVAL)) {
      odoTimerElapsed = true;
  }

  // --- Perform Odometer Save if Pending ---
  if (odoSavePending) {
      saveOdometer();
  }

  ////////BUTTON LOGIC (Short/Long Press Detection)////////
  if (digitalRead(2) == LOW) {
      if (!buttonIsDown) {
          buttonIsDown = true; buttonDownTime = currentMillis; longPressActive = false;
      } else if (!longPressActive && (currentMillis - buttonDownTime >= LONG_PRESS_DURATION)) {
          longPressTrigger = true; longPressActive = true; lastDataUpdateTime = currentMillis;
      }
  } else {
      if (buttonIsDown) {
          buttonIsDown = false;
          if (!longPressActive) { shortPressTrigger = true; lastDataUpdateTime = currentMillis; }
          longPressActive = false;
      }
  }

  // --- Process Button Actions Based on Current Screen State ---
  bool forceFullRedraw = false;

  if (currentState == SETTINGS_STATE) {
      // Settings Menu Actions
      if (shortPressTrigger) {
          settingsFocus = (settingsFocus + 1) % NUM_SETTINGS_ITEMS;
          if (settingsFocus == RESERVED_FOCUS_INDEX) settingsFocus = (settingsFocus + 1) % NUM_SETTINGS_ITEMS; // Skip reserved
          forceFullRedraw = true;
      } else if (longPressTrigger) {
          bool settingChanged = false;
          switch (settingsFocus) {
              case 0: if (currentUnits != 1) { currentUnits = 1; settingChanged = true; } break;
              case 1: if (currentUnits != 0) { currentUnits = 0; settingChanged = true; } break;
              case 2: if (currentRotation != 0) { currentRotation = 0; settingChanged = true; lcd.setRotation(currentRotation); } break;
              case 3: if (currentRotation != 1) { currentRotation = 1; settingChanged = true; lcd.setRotation(currentRotation); } break;
              case 4: if (currentRotation != 2) { currentRotation = 2; settingChanged = true; lcd.setRotation(currentRotation); } break;
              case 5: if (currentRotation != 3) { currentRotation = 3; settingChanged = true; lcd.setRotation(currentRotation); } break;
              case CLEAR_ODO_FOCUS_INDEX:
                  odometer = 0.0f; lastDistanceOdo = distance; saveOdometer(); settingChanged = true; break;
              case EXIT_FOCUS_INDEX:
                  currentState = 1;
                  if(odoTimerElapsed && velocity < ODO_SAVE_SPEED_THRESHOLD_KMH) saveOdometer();
                  settingChanged = true; forceFullRedraw = true; // Force redraw when exiting settings
                  break;
          }
          if (settingChanged) {
              EEPROM.write(EEPROM_ADDR_UNITS, currentUnits);
              EEPROM.write(EEPROM_ADDR_ROTATION, currentRotation);
              // EEPROM.commit();
              updateDisplayValues();
              forceFullRedraw = true; // Force redraw after setting change
          }
      }
  } else {
      // Normal Display Screen Actions
      if (shortPressTrigger) {
          currentState = (currentState % DISPLAY_STATE_MAX) + 1; // Cycle 1->2->3->4->1
          forceFullRedraw = true;
      } else if (longPressTrigger) {
          saveOdometer(); // Safety save before entering settings
          currentState = SETTINGS_STATE; settingsFocus = 0; forceFullRedraw = true;
      }
  }

  // --- Screen Wake Up Logic ---
  bool screenJustWoke = false;
  if (isScreenOff && (vescDataUpdated || shortPressTrigger || longPressTrigger)) {
      isScreenOff = false; screenJustWoke = true; forceFullRedraw = true;
  }

  // --- Screen Timeout Check ---
  if (!isScreenOff && currentMillis - lastDataUpdateTime > screenTimeoutDuration) {
      isScreenOff = true; saveOdometer(); lcd.fillScreen(BLACK); return;
  }

  // --- DISPLAY UPDATE LOGIC ---
  bool stateChanged = (currentState != lastState);
  bool fullRedrawNeeded = stateChanged || screenJustWoke || forceFullRedraw;
  bool updateTimeElapsed = (currentMillis - lastScreenUpdateTime >= screenUpdateInterval); // Use updated interval

  bool needsRedraw = !isScreenOff &&
                     ( (vescDataUpdated && updateTimeElapsed && currentState != SETTINGS_STATE) || fullRedrawNeeded );

  if (needsRedraw) {
      if (fullRedrawNeeded) {
          lcd.fillScreen(BLACK);
          lastDispVelocity1 = UNINITIALIZED_FLOAT; // Reset speed tracking on full redraw
      }

      if (currentState == SETTINGS_STATE) {
          drawSettingsScreen(); // Settings screen handles its own updates
      } else {
          // Draw the current screen
          // Conditional drawing is now only handled *within* drawScreen1 for speed
          // Other screens draw all elements each time they are called
          switch (currentState) {
              case 1: drawScreen1(); break;
              case 2: drawScreen2(); break;
              case 3: drawScreen3(); break;
              case 4: drawScreen4(); break;
          }
      }
      lastState = currentState;
      lastScreenUpdateTime = currentMillis;
  }
} // END OF loop()


// --- Unit Conversion Function ---
void updateDisplayValues() {
    if (currentUnits == 1) { // Standard Units
        displayVelocity = velocity * KM_TO_MILES;
        displayDistance = distance * KM_TO_MILES;
        displayOdometer = odometer * KM_TO_MILES;
        displayTempFet = (temp_fet * 9.0 / 5.0) + 32.0;
        displayTempMotor = (temp_motor * 9.0 / 5.0) + 32.0;
        displayMaxFetTemp = (maxfetemp * 9.0 / 5.0) + 32.0;
        displayMaxMotorTemp = (maxmotortemp * 9.0 / 5.0) + 32.0;
        if (displayDistance > 0.01) { displayWhPerDist = watthour / displayDistance; }
        else { displayWhPerDist = 0.0; }
        strcpy(velocityUnitStr, "mph"); strcpy(distanceUnitStr, "mi"); strcpy(odoUnitStr, "mi");
        strcpy(tempUnitStr, "F"); strcpy(consumptionUnitStr, "Wh/mi");
    } else { // Metric Units
        displayVelocity = velocity; displayDistance = distance; displayOdometer = odometer;
        displayTempFet = temp_fet; displayTempMotor = temp_motor;
        displayMaxFetTemp = maxfetemp; displayMaxMotorTemp = maxmotortemp;
        displayWhPerDist = whkm;
        strcpy(velocityUnitStr, "km/h"); strcpy(distanceUnitStr, "km"); strcpy(odoUnitStr, "km");
        strcpy(tempUnitStr, "C"); strcpy(consumptionUnitStr, "Wh/km");
    }
}


// --- Drawing Functions ---

// Helper function to draw a horizontal dotted line
void drawDottedLine(int x0, int y, int x1, uint16_t color, int spacing) {
    if (x0 > x1) { int temp = x0; x0 = x1; x1 = temp; }
    for (int x = x0; x <= x1; x += spacing) {
        lcd.drawPixel(x, y, color);
    }
}

// --- Screen 1: Main Telemetry Display ---
// Only speed display uses conditional drawing for flicker reduction
void drawScreen1() {
  int textHeight = 16; // Size 2 height
  lcd.setTextSize(2);

  // --- Battery indicator outline (Always redraw) ---
  lcd.drawRect(180, 2, 50, 25, WHITE);
  lcd.fillRect(175, 10, 5, 10, WHITE);

  // --- Trip Distance (Always redraw) ---
  lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
  lcd.setCursor(10, 10); lcd.print(displayDistance, 2);
  lcd.setTextColor(DEFAULT_UNIT_COLOR);
  lcd.print(" "); lcd.print(distanceUnitStr); lcd.print("   ");

  // --- Battery Percentage (Always redraw) ---
  lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
  lcd.setCursor(186, 8);
  if (batpercentage < 100) lcd.print(" ");
  if (batpercentage < 10) lcd.print(" ");
  lcd.print(batpercentage, 0);
  lcd.setTextColor(DEFAULT_UNIT_COLOR);
  lcd.print(F("%"));

  // --- Conditional Speed Display ---
  int currentSpeedInt = round(displayVelocity * 10);
  int lastSpeedInt = round(lastDispVelocity1 * 10);
  if (currentSpeedInt != lastSpeedInt || lastDispVelocity1 == UNINITIALIZED_FLOAT) {
      lcd.fillRect(0, velocity_y, SCR_WD, velocity_height, BLACK); // Clear area
      lcd.setCursor(10, velocity_y);
      lcd.setTextSize(9);
      lcd.setTextColor(DEFAULT_VALUE_COLOR); // No BG needed after fillRect
      if (displayVelocity < 10.0 && displayVelocity >= 0) lcd.print(" ");
      lcd.print(displayVelocity, 1);
      lastDispVelocity1 = displayVelocity; // Update last drawn value
  }
  lcd.setTextSize(2); // Reset text size

  // --- Data Rows Below Velocity ---
  int currentY = velocity_y + velocity_height + 10;
  int rowHeight = textHeight + rowSeparatorMargin * 2;
  int separatorY;

  // Helper lambda for drawing rows (uses background clearing)
  auto drawDataRow = [&](const __FlashStringHelper* label, float value, int precision, const char* unit,
                         bool isInt, bool checkNegative)
  {
      uint16_t currentLabelColor = DEFAULT_LABEL_COLOR;
      uint16_t currentUnitColor = DEFAULT_UNIT_COLOR;
      if (checkNegative && value < 0) { // Check for negative value
          currentLabelColor = REGEN_COLOR;
          currentUnitColor = REGEN_COLOR;
      }

      // Draw separator (always draw for simplicity)
      separatorY = currentY - rowSeparatorMargin;
      if (separatorY > (velocity_y + velocity_height)) {
          drawDottedLine(separatorStartX, separatorY, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
      }

      // Draw Label (with background clearing)
      lcd.setTextColor(currentLabelColor, BLACK);
      lcd.setCursor(labelStartX1, currentY); lcd.print(label);

      // Draw Value (with background clearing)
      lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
      lcd.setCursor(valueStartX1, currentY);
      if (isInt) lcd.print((int)round(value)); else lcd.print(value, precision);
      lcd.print(F("   ")); // Padding

      // Draw Unit (with background clearing)
      lcd.setTextColor(currentUnitColor, BLACK);
      lcd.setCursor(unitStartX1, currentY); lcd.print(unit); lcd.print(F("   ")); // Padding for unit

      currentY += rowHeight; // Increment Y position
  };

  // Draw the rows
  drawDataRow(F("Power"), power, 0, " W", true, true); // Check power for negative
  if (CELL_COUNT > 0) {
      drawDataRow(F("Cell V"), voltage / (float)CELL_COUNT, 2, " V", false, false);
  } else {
      lcd.fillRect(0, currentY, SCR_WD, rowHeight, BLACK); // Clear row if skipping
      currentY += rowHeight;
  }
  drawDataRow(F("FET Temp"), displayTempFet, 0, tempUnitStr, true, false);
  drawDataRow(F("Mot Temp"), displayTempMotor, 0, tempUnitStr, true, false);
}


// --- Screen 2: Detailed Data Display ---
// Simplified: Redraws all elements each time using background clearing
void drawScreen2() {
    int y = 15; // Starting Y position
    const int textHeight = 16;
    const int textSize = 2;
    const int rowHeight = textHeight + rowSeparatorMargin * 2;

    // Helper lambda for drawing lines (handles regen color, uses BG clearing)
    auto printDataLine = [&](const __FlashStringHelper* label, float value, int precision, const char* unit,
                             bool isInt, bool checkNegative)
    {
        uint16_t currentLabelColor = DEFAULT_LABEL_COLOR;
        uint16_t currentUnitColor = DEFAULT_UNIT_COLOR;
        if (checkNegative && value < 0) {
            currentLabelColor = REGEN_COLOR;
            currentUnitColor = REGEN_COLOR;
        }

        // Draw separator
        int separatorY = y - rowSeparatorMargin;
        if (separatorY > 5) {
            drawDottedLine(separatorStartX, separatorY, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
        }
        lcd.setTextSize(textSize);

        // Label (with background clearing)
        lcd.setTextColor(currentLabelColor, BLACK);
        lcd.setCursor(labelStartX2, y); lcd.print(label); lcd.print(F("      ")); // Padding

        // Value (with background clearing)
        lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
        lcd.setCursor(valueStartX2, y);
        if (isInt) lcd.print((int)round(value)); else lcd.print(value, precision);
        lcd.print(F("    ")); // Padding

        // Unit (with background clearing)
        lcd.setTextColor(currentUnitColor, BLACK);
        lcd.setCursor(unitStartX2, y); lcd.print(unit); lcd.print(F("   ")); // Padding

        y += rowHeight; // Increment Y position
    };

    // --- Draw items in the new order ---
    printDataLine(F("Power"), power, 0, "W", true, true); // Check power
    printDataLine(F("Batt A"), current, 1, "A", false, true); // Check batt current
    printDataLine(F("Phase A"), phase_current, 1, "A", false, true); // Check phase current
    printDataLine(F("Wh used"), watthour, 1, "Wh", false, false);
    printDataLine(F("Ah used"), amphour, 2, "Ah", false, false);
    if (CELL_COUNT > 0) {
        printDataLine(F("Cell V"), voltage / (float)CELL_COUNT, 2, "V", false, false);
    } else {
        lcd.fillRect(0, y, SCR_WD, rowHeight, BLACK); // Clear row if skipping
        y += rowHeight;
    }
    printDataLine(F("RPM"), rpm, 0, "", true, false);
    printDataLine(F("Mot Temp"), displayTempMotor, 0, tempUnitStr, true, false);
    printDataLine(F("FET Temp"), displayTempFet, 0, tempUnitStr, true, false);
}


// --- Screen 3: Temperature Focus Screen ---
// Simplified: Redraws all elements each time (large font = potential flicker)
void drawScreen3() {
  // Title and Separator
  int titleY = 10;
  int titleHeight = 16;
  int separatorY = titleY + titleHeight + 5;
  lcd.setTextColor(White, BLACK);
  lcd.setTextSize(2);
  int titleX = 35;
  lcd.setCursor(titleX, titleY);
  if (currentUnits == 1) { lcd.print(F("TEMPERATURE (F)")); }
  else { lcd.print(F("TEMPERATURE (C)")); }
  lcd.fillRect(0, separatorY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR);

  // Temp Data (Large Font)
  lcd.setTextSize(3);
  int currentY = separatorY + titleSeparatorHeight + 15;
  const int textHeight = 24; // Size 3 height
  const int rowHeight = textHeight + rowSeparatorMargin * 2;

  // Helper lambda for temp rows (using BG clearing for simplicity here)
  auto drawTempRow = [&](const __FlashStringHelper* label, float value) {
      int sepY_draw = currentY - rowSeparatorMargin;
      if (sepY_draw > separatorY + titleSeparatorHeight) {
          drawDottedLine(separatorStartX, sepY_draw, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
      }
      // Using text with background clearing for Screen 3 for now
      lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
      lcd.setCursor(labelStartX3, currentY); lcd.print(label); lcd.print(F("      "));
      lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
      lcd.setCursor(valueStartX3, currentY); lcd.print((int)round(value)); lcd.print(F("    "));
      currentY += rowHeight;
  };

  // Draw the rows
  drawTempRow(F("Mot"), displayTempMotor);
  drawTempRow(F("Mot Max"), displayMaxMotorTemp);
  drawTempRow(F("FET"), displayTempFet);
  drawTempRow(F("FET Max"), displayMaxFetTemp);
}


// --- Screen 4: Trip Information Screen ---
// Simplified: Redraws all elements each time (using BG clearing)
void drawScreen4() {
    // Title and Separator
    int titleY = 10;
    int titleHeight = 16;
    int separatorY = titleY + titleHeight + 5;
    lcd.setTextColor(White, BLACK);
    lcd.setTextSize(2);
    lcd.setCursor(60, titleY); lcd.print(F("TRIP INFO"));
    lcd.fillRect(0, separatorY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR);

    // Trip Data
    lcd.setTextSize(2);
    int currentY = separatorY + titleSeparatorHeight + 10;
    int labelX = 15;
    int valueX = 125;
    int unitX = 175;
    const int textHeight = 16;
    const int rowSpacing = 8;
    const int rowHeight = textHeight + rowSpacing;

    // Helper lambda for trip data lines (using BG clearing)
    auto drawTripLine = [&](const __FlashStringHelper* label, float value, int precision, const char* unit) {
        int sepY_draw = currentY - (rowSpacing / 2) - 1 ;
        if (currentY > separatorY + titleSeparatorHeight + 5) {
             drawDottedLine(separatorStartX, sepY_draw, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
        }
        lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
        lcd.setCursor(labelX, currentY); lcd.print(label); lcd.print(" ");
        lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
        lcd.setCursor(valueX, currentY); lcd.print(value, precision); lcd.print(" ");
        lcd.setTextColor(DEFAULT_UNIT_COLOR, BLACK);
        lcd.setCursor(unitX, currentY); lcd.print(unit); lcd.print("  ");
        currentY += rowHeight;
    };

    // Draw the lines
    drawTripLine(F("ODO"), displayOdometer, 1, odoUnitStr);
    drawTripLine(F("Dist"), displayDistance, 2, distanceUnitStr);
    drawTripLine(F("Wh Used"), watthour, 1, "Wh");
    drawTripLine(F("Avg Cons"), displayWhPerDist, 1, consumptionUnitStr);
}


// --- Screen 5: Settings Screen ---
// Settings screen redraws fully on focus change anyway
void drawSettingsScreen() {
    lcd.setTextSize(2);
    int currentY = 10;
    int titleHeight = 16;
    int itemHeight = 16;
    int itemSpacing = 10;
    int lineY;
    int textW; // Calculated text width

    // Title
    lcd.setTextColor(White, BLACK);
    lcd.setCursor(70, currentY);
    lcd.print(F("SETTINGS"));
    currentY += titleHeight + 5;
    lcd.fillRect(0, currentY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR); // Title separator
    currentY += titleSeparatorHeight + 15;

    // Units Label
    lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
    lcd.setCursor(settingsLabelX, currentY);
    lcd.print(F("Units"));
    currentY += itemHeight + itemSpacing;

    // Units Options (Standard / Metric)
    int unitsOptionX = settingsUnitsOptionX;

    // Standard Option
    const char* standardStr = "Standard";
    textW = strlen(standardStr) * 6 * 2; // Approx width
    lcd.setCursor(unitsOptionX, currentY);
    if (settingsFocus == 0) { // Focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR); // Draw BG
        lcd.setTextColor(SETTINGS_FOCUS_COLOR); // Set text color
    } else { // Not focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK); // Clear BG
        lcd.setTextColor(SETTINGS_ITEM_COLOR); // Set text color
    }
    lcd.print(standardStr); // Draw text
    lineY = currentY + itemHeight + 1; // Underline position
    if (currentUnits == 1) { lcd.fillRect(unitsOptionX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); } // Draw underline if selected
    else { lcd.fillRect(unitsOptionX, lineY, textW, 1, BLACK); } // Clear underline if not selected
    unitsOptionX += settingsUnitsOptionSpacing; // Move X for next option

    // Metric Option
    const char* metricStr = "Metric";
    textW = strlen(metricStr) * 6 * 2;
    lcd.setCursor(unitsOptionX, currentY);
    if (settingsFocus == 1) { // Focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
        lcd.setTextColor(SETTINGS_ITEM_COLOR);
    }
    lcd.print(metricStr);
    lineY = currentY + itemHeight + 1;
    if (currentUnits == 0) { lcd.fillRect(unitsOptionX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); }
    else { lcd.fillRect(unitsOptionX, lineY, textW, 1, BLACK); }
    currentY += itemHeight + itemSpacing + 5; // Move Y down

    // Screen Rotation Label
    lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
    lcd.setCursor(settingsLabelX, currentY);
    lcd.print(F("Screen Rotation"));
    currentY += itemHeight + itemSpacing;

    // Rotation Options (0, 90, 180, 270)
    int rotOptionsX = settingsRotOptionX;
    const char* rotLabels[] = {"0", "90", "180", "270"};
    for (int i = 0; i < 4; i++) {
        int labelLen = strlen(rotLabels[i]);
        textW = labelLen * 6 * 2; // Approx width
        lcd.setCursor(rotOptionsX, currentY);
        if (settingsFocus == (i + 2)) { // Focused
             lcd.fillRect(rotOptionsX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
             lcd.setTextColor(SETTINGS_FOCUS_COLOR);
        } else { // Not focused
             lcd.fillRect(rotOptionsX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
             lcd.setTextColor(SETTINGS_ITEM_COLOR);
        }
        lcd.print(rotLabels[i]); // Draw text
        lineY = currentY + itemHeight + 1; // Underline position
        if (currentRotation == i) { lcd.fillRect(rotOptionsX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); } // Draw underline if selected
        else { lcd.fillRect(rotOptionsX, lineY, textW, 1, BLACK); } // Clear underline
        rotOptionsX += settingsRotOptionSpacing; // Move X for next option
    }
    currentY += itemHeight + itemSpacing + 5; // Move Y down

    // Clear Odometer Option
    const char* clearOdoStr = "CLEAR ODO";
    textW = strlen(clearOdoStr) * 6 * 2;
    lcd.setCursor(settingsLabelX, currentY);
    if (settingsFocus == CLEAR_ODO_FOCUS_INDEX) { // Focused
        lcd.fillRect(settingsLabelX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(settingsLabelX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
        lcd.setTextColor(DEFAULT_LABEL_COLOR); // Use DEFAULT label color when not focused
    }
    lcd.print(clearOdoStr); // Draw text
    currentY += itemHeight + itemSpacing + 10; // Move Y down

    // EXIT Option (Large Font)
    lcd.setTextSize(3);
    const char* exitStr = "EXIT";
    int exitTextW = strlen(exitStr) * 6 * 3; // Approx width for size 3
    int exitX = (SCR_WD - exitTextW) / 2; // Center horizontally
    int exitY = SCR_HT - 30; // Position near bottom
    int exitItemH = 24; // Height for size 3
    lcd.setCursor(exitX, exitY);
    if (settingsFocus == EXIT_FOCUS_INDEX) { // Focused
        lcd.fillRect(exitX - 2, exitY - 1, exitTextW + 4, exitItemH + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(exitX - 2, exitY - 1, exitTextW + 4, exitItemH + 2, BLACK);
        lcd.setTextColor(DEFAULT_LABEL_COLOR); // Use DEFAULT label color when not focused
    }
    lcd.print(exitStr); // Draw text
}

// --- END OF FILE ---
